@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-# Copyright (C) 2015-2019 Damon Lynch <damonlynch@gmail.com>
+# Copyright (C) 2015-2020 Damon Lynch <damonlynch@gmail.com>
 # Copyright (C) 2012-2015 Jim Easterbrook <jim@jim-easterbrook.me.uk>
 
 # This file is part of Rapid Photo Downloader.
@@ -20,14 +20,14 @@
 # see <http://www.gnu.org/licenses/>.
 
 __author__ = 'Damon Lynch'
-__copyright__ = "Copyright 2015-2019, Damon Lynch. Copyright 2012-2015 Jim Easterbrook."
+__copyright__ = "Copyright 2015-2020, Damon Lynch. Copyright 2012-2015 Jim Easterbrook."
 
 import logging
 import os
 import io
 from collections import namedtuple
 import re
-from typing import Optional, List, Tuple
+from typing import Optional, List, Tuple, Union
 
 import gphoto2 as gp
 from raphodo.storage import StorageSpace
@@ -38,10 +38,46 @@ from raphodo.utilities import format_size_for_user
 def python_gphoto2_version():
     return  gp.__version__
 
+
 def gphoto2_version():
     return gp.gp_library_version(0)[0]
 
 
+def gphoto2_python_logging():
+    """
+    Version 2.0.0 of gphoto2 introduces a COMPATIBILITY CHANGE:
+    gp_log_add_func and use_python_logging now return a
+    Python object which must be stored until logging is no longer needed.
+    Could just go with the None returned by default from a function that
+    returns nothing, but want to make this explicit.
+
+    :return: either True or a Python object that must be stored until logging
+     is no longer needed
+    """
+
+    return gp.use_python_logging() or True
+
+
+def autodetect_cameras(context: gp.Context,
+                       suppress_errors: bool=True) -> Union[gp.CameraList, List]:
+    """
+    Do camera auto detection for multiple versions of gphoto2-python
+
+    Version 2.2.0 of gphoto2 introduces a COMPATIBILITY CHANGE:
+    Removed Context.camera_autodetect method.
+    Was quickly reintroduced in 2.2.1, but is due for removal.
+
+    :return: CameraList of model and port
+    """
+
+    try:
+        return gp.check_result(gp.gp_camera_autodetect(context))
+    except Exception:
+        if not suppress_errors:
+            raise
+        return []
+
+
 # convert error codes to error names
 gphoto2_error_codes = {
     code: name for code, name in (
@@ -132,7 +168,8 @@ class Camera:
         """
         Initialize a camera via libgphoto2.
 
-        :param model: camera model, as returned by camera_autodetect()
+        :param model: camera model, as returned by camera_autodetect() or
+         gp_camera_autodetect()
         :param port: camera port, as returned by camera_autodetect()
         :param get_folders: whether to detect the DCIM folders on the
          camera
@@ -158,7 +195,7 @@ class Camera:
 
         self._select_camera(model, port)
 
-        self.specific_folders = None # type: List[str]
+        self.specific_folders = None  # type: Optional[List[str]]
         self.specific_folder_located = False
         self._dual_slots_active = False
 
@@ -192,6 +229,11 @@ class Camera:
                     path='/', specific_folders=specific_folders
                 )
                 self.specific_folder_located = len(self.specific_folders) > 0
+
+                logging.debug(
+                    "Folders located on %s: %s", self.display_name,
+                    ', '.join(', '.join(map(str, sl)) for sl in self.specific_folders)
+                )
             except gp.GPhoto2Error as e:
                 logging.error(
                     "Unable to access camera %s: %s. Is it locked?",
@@ -207,7 +249,7 @@ class Camera:
         abilities = self.camera.get_abilities()
         self.can_fetch_thumbnails = abilities.file_operations & gp.GP_FILE_OPERATION_PREVIEW != 0
 
-    def camera_has_dcim_like_folder(self) -> bool:
+    def camera_has_folders_to_scan(self) -> bool:
         """
         Check whether the camera has been initialized and if a DCIM or other specific folder
         has been located
@@ -822,11 +864,10 @@ class Camera:
             return True
 
 
-
 def dump_camera_details() -> None:
     import itertools
     context = gp.Context()
-    cameras = context.camera_autodetect()
+    cameras = autodetect_cameras(context)
     for model, port in cameras:
         c = Camera(model=model, port=port, context=context)
         if not c.camera_initialized:
@@ -873,8 +914,10 @@ def dump_camera_details() -> None:
 
 
 if __name__ == "__main__":
+    print("gphoto2 python: ", python_gphoto2_version())
+    # logging = gphoto2_python_logging()
 
-    if False:
+    if True:
         dump_camera_details()
 
     if True:
@@ -882,7 +925,7 @@ if __name__ == "__main__":
         #Test stub
         gp_context = gp.Context()
         # Assume gphoto2 version 2.5 or greater
-        cameras = gp_context.camera_autodetect()
+        cameras = autodetect_cameras(gp_context)
         for name, value in cameras:
             camera = name
             port = value
