@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-# Copyright (C) 2011-2019 Damon Lynch <damonlynch@gmail.com>
+# Copyright (C) 2011-2020 Damon Lynch <damonlynch@gmail.com>
 
 # This file is part of Rapid Photo Downloader.
 #
@@ -43,7 +43,7 @@ A sample photo or video for (1) can be used for (2)
 
 """
 __author__ = 'Damon Lynch'
-__copyright__ = "Copyright 2011-2019, Damon Lynch"
+__copyright__ = "Copyright 2011-2020, Damon Lynch"
 
 import os
 import sys
@@ -54,8 +54,11 @@ from datetime import datetime
 import tempfile
 import operator
 import locale
-# Use the default locale as defined by the LANG variable
-locale.setlocale(locale.LC_ALL, '')
+try:
+    # Use the default locale as defined by the LANG variable
+    locale.setlocale(locale.LC_ALL, '')
+except locale.Error:
+    pass
 
 if sys.version_info < (3,5):
     import scandir
@@ -73,7 +76,9 @@ from raphodo.preferences import ScanPreferences, Preferences
 from raphodo.interprocess import (
     WorkerInPublishPullPipeline, ScanResults, ScanArguments
 )
-from raphodo.camera import Camera, CameraError, CameraProblemEx
+from raphodo.camera import (
+    Camera, CameraError, CameraProblemEx, gphoto2_python_logging, gphoto2_named_error
+)
 import raphodo.rpdfile as rpdfile
 from raphodo.constants import (
     DeviceType, FileType, DeviceTimestampTZ, CameraErrorCode, FileExtension,
@@ -82,7 +87,8 @@ from raphodo.constants import (
 from raphodo.rpdsql import DownloadedSQL, FileDownloaded
 from raphodo.cache import ThumbnailCacheSql
 from raphodo.utilities import (
-    stdchannel_redirected, datetime_roughly_equal, GenerateRandomFileName, format_size_for_user
+    stdchannel_redirected, datetime_roughly_equal, GenerateRandomFileName, format_size_for_user,
+    is_snap
 )
 from raphodo.exiftool import ExifTool
 import raphodo.metadatavideo as metadatavideo
@@ -195,7 +201,7 @@ class ScanWorker(WorkerInPublishPullPipeline):
 
         scan_arguments = pickle.loads(self.content)  # type: ScanArguments
         if scan_arguments.log_gphoto2:
-            gp.use_python_logging()
+            self.gphoto2_logging = gphoto2_python_logging()
 
         if scan_arguments.ignore_other_types:
             fileformats.PHOTO_EXTENSIONS_SCAN = fileformats.PHOTO_EXTENSIONS_WITHOUT_OTHER
@@ -219,180 +225,52 @@ class ScanWorker(WorkerInPublishPullPipeline):
 
         self.files_scanned = 0
         self.camera = None
+        terminated = False
 
         if not self.download_from_camera:
-            # Download from file system - either on This Computer, or an external volume like a
-            # memory card or USB Flash or external drive of some kind
-            path = os.path.abspath(scan_arguments.device.path)
-            self.display_name = scan_arguments.device.display_name
-
-            scanning_specific_path = self.prefs.scan_specific_folders and \
-                            scan_arguments.device.device_type == DeviceType.volume
-            if scanning_specific_path:
-                specific_folder_prefs = self.prefs.folders_to_scan
-                paths = tuple(
-                    os.path.join(path, folder) for folder in os.listdir(path)
-                    if folder in specific_folder_prefs and os.path.isdir(os.path.join(path, folder))
-                )
-                logging.info(
-                    "For device %s, identified paths: %s", self.display_name, ', '.join(paths)
-                )
-            else:
-                paths = path,
-
-            if scan_arguments.device.device_type == DeviceType.volume:
-                device_type = 'device'
-            else:
-                device_type = 'This Computer path'
-            logging.info("Scanning {} {}".format(device_type, self.display_name))
-
-            self.problems.uri = get_uri(path=path)
-            self.problems.name = self.display_name
-
-            # Before doing anything else, determine time zone approach
-            # Need two different walks because first folder of files
-            # might be videos, then the 2nd folder photos, etc.
-            for path in paths:
-                self.distinguish_non_camera_device_timestamp(path)
-                if self.device_timestamp_type != DeviceTimestampTZ.undetermined:
-                    break
-
-            for path in paths:
-                if scanning_specific_path:
-                    logging.info("Scanning {} on {}".format(path, self.display_name))
-                for dir_name, name in self.walk_file_system(path):
-                    self.dir_name = dir_name
-                    self.file_name = name
-                    self.process_file()
-
+            self.scan_file_system(scan_arguments)
         else:
-            # scanning directly from camera
-            have_optimal_display_name = scan_arguments.device.have_optimal_display_name
-            if self.prefs.scan_specific_folders:
-                specific_folder_prefs =  self.prefs.folders_to_scan
-            else:
-                specific_folder_prefs = None
-            while True:
+            try:
+                self.scan_camera(scan_arguments)
+                # Sanity check: ensure file contents are still accessible
                 try:
-                    self.camera = Camera(
-                        model=scan_arguments.device.camera_model,
-                        port=scan_arguments.device.camera_port,
-                        raise_errors=True,
-                        specific_folders=specific_folder_prefs
-                    )
-                    if not have_optimal_display_name:
-                        # Update the GUI with the real name of the camera
-                        # and its storage information
-                        have_optimal_display_name = True
-                        self.camera_display_name = self.camera.display_name
-                        self.display_name = self.camera_display_name
-                        storage_space = self.camera.get_storage_media_capacity(refresh=True)
-                        storage_descriptions = self.camera.get_storage_descriptions()
-                        self.content = pickle.dumps(
-                            ScanResults(
-                                optimal_display_name=self.camera_display_name,
-                                storage_space=storage_space,
-                                storage_descriptions=storage_descriptions,
-                                scan_id=int(self.worker_id),
-                            ),
-                            pickle.HIGHEST_PROTOCOL
-                        )
-                        self.send_message_to_sink()
-                    break
-                except CameraProblemEx as e:
-                    self.content = pickle.dumps(
-                        ScanResults(
-                            error_code=e.code, scan_id=int(self.worker_id)
-                        ),
-                        pickle.HIGHEST_PROTOCOL
-                    )
-                    self.send_message_to_sink()
-                    # Wait for command to resume or halt processing
-                    self.resume_work()
+                    self.camera.camera.folder_list_files('/', self.camera.context)
+                except gp.GPhoto2Error as e:
+                    raise CameraError(CameraErrorCode.inaccessible)
+                else:
+                    self.camera.free_camera()
 
-            if self.download_from_camera:
-                self.camera_details = 0
-            self.problems.uri = get_uri(camera_details=self.camera_details)
-            self.problems.name = self.display_name
+            except CameraError as e:
+                if e.code == CameraErrorCode.inaccessible:
+                    terminated = True
+                    logging.info("Terminating scan of %s", self.display_name)
+                    if self.is_mtp_device:
+                        logging.debug("%s is an MTP device", self.display_name)
+                    if self.camera is not None:
+                        self.camera.free_camera()
+                else:
+                    raise
 
-            if self.ignore_mdatatime_for_mtp_dng:
-                logging.info(
-                    "For any DNG files on the %s, when determining the creation date/"
-                    "time, the metadata date/time will be ignored, and the file "
-                    "modification date/time used instead", self.display_name
+        if not terminated:
+            if self.file_batch:
+                # Send any remaining files, including the sample photo or video
+                self.content = pickle.dumps(
+                    ScanResults(
+                        self.file_batch,
+                        self.file_type_counter,
+                        self.file_size_sum,
+                        sample_photo=self.sample_photo,
+                        sample_video=self.sample_video,
+                        entire_video_required=self.entire_video_required,
+                        entire_photo_required=self.entire_photo_required,
+                    ),
+                    pickle.HIGHEST_PROTOCOL
                 )
-
-            # Download only from the DCIM folder(s) in the camera.
-            # Phones especially have many directories with images, which we
-            # must ignore
-            if self.camera.camera_has_dcim_like_folder():
-                logging.info("Scanning {}".format(self.display_name))
-                self._camera_folders_and_files = []
-                self._camera_file_names = defaultdict(list)
-                self._camera_audio_files = defaultdict(list)
-                self._camera_video_thumbnails = defaultdict(list)
-                self._camera_xmp_files = defaultdict(list)
-                self._camera_log_files = defaultdict(list)
-                self._folder_identifiers = {}
-                self._folder_identifers_for_file = \
-                    defaultdict(list) # type: DefaultDict[int, List[int]]
-                self._camera_directories_for_file = defaultdict(list)
-                self._camera_photos_videos_by_type = \
-                    defaultdict(list)  # type: DefaultDict[FileExtension, List[CameraMetadataDetails]]
-
-                specific_folders = self.camera.specific_folders
-
-                if self.camera.dual_slots_active:
-                    # This camera has dual memory cards in use.
-                    # Give each folder an numeric identifier that will be
-                    # used to identify which card a given file comes from
-                    for idx, folders in enumerate(specific_folders):
-                        for folder in folders:
-                            self._folder_identifiers[folder] = idx + 1
-
-                # locate photos and videos, identifying duplicate files
-                # identify candidates for extracting metadata
-                for idx, folders in enumerate(specific_folders):
-                    # Setup camera details for each storage space in the camera
-                    self.camera_details = idx
-                    # Now initialize the problems container, if not already done so
-                    if idx:
-                        self.problems.name = self.camera_display_name
-                        self.problems.uri = get_uri(camera_details=self.camera_details)
-
-                    for specific_folder in folders:
-                        logging.debug(
-                            "Scanning %s on %s", specific_folder, self.camera.display_name
-                        )
-                        folder_identifier = self._folder_identifiers.get(specific_folder)
-                        basedir = os.path.dirname(specific_folder)
-                        self.locate_files_on_camera(specific_folder, folder_identifier, basedir)
-
-                # extract non camera metadata
-                if self._camera_photos_videos_by_type:
-                    self.identify_camera_tz_and_sample_files()
-
-                # now, process each file
-                for self.dir_name, self.file_name in self._camera_folders_and_files:
-                    self.process_file()
-            else:
-                logging.warning(
-                    "Unable to detect any specific folders (like DCIM) on %s", self.display_name
-                )
-
-            self.camera.free_camera()
-
-        if self.file_batch:
-            # Send any remaining files, including the sample photo or video
+                self.send_message_to_sink()
+        elif self.download_from_camera:
             self.content = pickle.dumps(
                 ScanResults(
-                    self.file_batch,
-                    self.file_type_counter,
-                    self.file_size_sum,
-                    sample_photo=self.sample_photo,
-                    sample_video=self.sample_video,
-                    entire_video_required=self.entire_video_required,
-                    entire_photo_required=self.entire_photo_required,
+                    scan_id=int(self.worker_id), camera_removed=True
                 ),
                 pickle.HIGHEST_PROTOCOL
             )
@@ -440,6 +318,180 @@ class ScanWorker(WorkerInPublishPullPipeline):
             for name in file_list:
                 yield dir_name, name
 
+    def scan_file_system(self, scan_arguments: ScanArguments):
+        """
+        Download from file system - either on This Computer, or an external volume like a
+        # memory card or USB Flash or external drive of some kind
+
+        :param scan_arguments: scan configuration
+        """
+
+        path = os.path.abspath(scan_arguments.device.path)
+        self.display_name = scan_arguments.device.display_name
+
+        scanning_specific_path = self.prefs.scan_specific_folders and \
+                                 scan_arguments.device.device_type == DeviceType.volume
+        if scanning_specific_path:
+            specific_folder_prefs = self.prefs.folders_to_scan
+            paths = tuple(
+                os.path.join(path, folder) for folder in os.listdir(path)
+                if folder in specific_folder_prefs and os.path.isdir(os.path.join(path, folder))
+            )
+            logging.info(
+                "For device %s, identified paths: %s", self.display_name, ', '.join(paths)
+            )
+        else:
+            paths = path,
+
+        if scan_arguments.device.device_type == DeviceType.volume:
+            device_type = 'device'
+        else:
+            device_type = 'This Computer path'
+        logging.info("Scanning {} {}".format(device_type, self.display_name))
+
+        self.problems.uri = get_uri(path=path)
+        self.problems.name = self.display_name
+
+        # Before doing anything else, determine time zone approach
+        # Need two different walks because first folder of files
+        # might be videos, then the 2nd folder photos, etc.
+        for path in paths:
+            self.distinguish_non_camera_device_timestamp(path)
+            if self.device_timestamp_type != DeviceTimestampTZ.undetermined:
+                break
+
+        for path in paths:
+            if scanning_specific_path:
+                logging.info("Scanning {} on {}".format(path, self.display_name))
+            for dir_name, name in self.walk_file_system(path):
+                self.dir_name = dir_name
+                self.file_name = name
+                self.process_file()
+
+    def scan_camera(self, scan_arguments: ScanArguments) -> None:
+        """
+        Scan camera for files.
+
+        Raises error if camera becomes inaccessible
+
+        :param scan_arguments: scan configuration
+        """
+
+        have_optimal_display_name = scan_arguments.device.have_optimal_display_name
+        if self.prefs.scan_specific_folders:
+            specific_folder_prefs = self.prefs.folders_to_scan
+        else:
+            specific_folder_prefs = None
+        while True:
+            try:
+                self.camera = Camera(
+                    model=scan_arguments.device.camera_model,
+                    port=scan_arguments.device.camera_port,
+                    raise_errors=True,
+                    specific_folders=specific_folder_prefs
+                )
+                if not have_optimal_display_name:
+                    # Update the GUI with the real name of the camera
+                    # and its storage information
+                    have_optimal_display_name = True
+                    self.camera_display_name = self.camera.display_name
+                    self.display_name = self.camera_display_name
+                    storage_space = self.camera.get_storage_media_capacity(refresh=True)
+                    storage_descriptions = self.camera.get_storage_descriptions()
+                    self.content = pickle.dumps(
+                        ScanResults(
+                            optimal_display_name=self.camera_display_name,
+                            storage_space=storage_space,
+                            storage_descriptions=storage_descriptions,
+                            scan_id=int(self.worker_id),
+                        ),
+                        pickle.HIGHEST_PROTOCOL
+                    )
+                    self.send_message_to_sink()
+                break
+            except CameraProblemEx as e:
+                self.content = pickle.dumps(
+                    ScanResults(
+                        error_code=e.code, scan_id=int(self.worker_id)
+                    ),
+                    pickle.HIGHEST_PROTOCOL
+                )
+                self.send_message_to_sink()
+                # Wait for command to resume or halt processing
+                self.resume_work()
+
+        self.camera_details = 0
+        self.problems.uri = get_uri(camera_details=self.camera_details)
+        self.problems.name = self.display_name
+
+        if self.ignore_mdatatime_for_mtp_dng:
+            logging.info(
+                "For any DNG files on the %s, when determining the creation date/"
+                "time, the metadata date/time will be ignored, and the file "
+                "modification date/time used instead", self.display_name
+            )
+
+        # Download only from the DCIM type folder(s) in the camera,
+        # if that's what the user has specified. Otherwise, try to download from everything we
+        # can find.
+        if self.camera.camera_has_folders_to_scan():
+            logging.info("Scanning {}".format(self.display_name))
+            self._camera_folders_and_files = []
+            self._camera_file_names = defaultdict(list)
+            self._camera_audio_files = defaultdict(list)
+            self._camera_video_thumbnails = defaultdict(list)
+            self._camera_xmp_files = defaultdict(list)
+            self._camera_log_files = defaultdict(list)
+            self._folder_identifiers = {}
+            self._folder_identifers_for_file = \
+                defaultdict(list)  # type: DefaultDict[int, List[int]]
+            self._camera_directories_for_file = defaultdict(list)
+            self._camera_photos_videos_by_type = \
+                defaultdict(list)  # type: DefaultDict[FileExtension, List[CameraMetadataDetails]]
+
+            specific_folders = self.camera.specific_folders
+
+            if self.camera.dual_slots_active:
+                # This camera has dual memory cards in use.
+                # Give each folder an numeric identifier that will be
+                # used to identify which card a given file comes from
+                for idx, folders in enumerate(specific_folders):
+                    for folder in folders:
+                        self._folder_identifiers[folder] = idx + 1
+
+            # locate photos and videos, identifying duplicate files
+            # identify candidates for extracting metadata
+            for idx, folders in enumerate(specific_folders):
+                # Setup camera details for each storage space in the camera
+                self.camera_details = idx
+                # Now initialize the problems container, if not already done so
+                if idx:
+                    self.problems.name = self.camera_display_name
+                    self.problems.uri = get_uri(camera_details=self.camera_details)
+
+                for specific_folder in folders:
+                    logging.debug(
+                        "Scanning %s on %s", specific_folder, self.camera.display_name
+                    )
+                    folder_identifier = self._folder_identifiers.get(specific_folder)
+                    if specific_folder_prefs is None:
+                        basedir = specific_folder
+                    else:
+                        basedir = os.path.dirname(specific_folder)
+                    self.locate_files_on_camera(specific_folder, folder_identifier, basedir)
+
+            # extract camera metadata
+            if self._camera_photos_videos_by_type:
+                self.identify_camera_tz_and_sample_files()
+
+            # now, process each file
+            for self.dir_name, self.file_name in self._camera_folders_and_files:
+                self.process_file()
+        else:
+            logging.warning(
+                "Unable to detect any specific folders (like DCIM) on %s", self.display_name
+            )
+
     def locate_files_on_camera(self, path: str, folder_identifier: int, basedir: str) -> None:
         """
         Scans the memory card(s) on the camera for photos, videos,
@@ -473,6 +525,8 @@ class ScanWorker(WorkerInPublishPullPipeline):
         For duplicate files, we record both directories the file is
         stored on.
 
+        We ignore all folders that contain a file .nomedia
+
         :param path: the path on the camera to analyze for files and
          folders
         :param folder_identifier: if not None, then indicates (1) the
@@ -488,13 +542,22 @@ class ScanWorker(WorkerInPublishPullPipeline):
         try:
             files_in_folder = self.camera.camera.folder_list_files(path, self.camera.context)
         except gp.GPhoto2Error as e:
-            logging.error("Unable to scan files on camera: error %s", e.code)
+            logging.error(
+                "Unable to scan files on %s: %s", self.display_name, gphoto2_named_error(e.code)
+            )
             uri = get_uri(path=path, camera_details=self.camera_details)
             self.problems.append(CameraDirectoryReadProblem(uri=uri, name=path, gp_code=e.code))
+            if e.code in (gp.GP_ERROR_IO_USB_FIND, gp.GP_ERROR_BAD_PARAMETERS):
+                logging.error("%s removed while listing files during scan", self.display_name)
+                raise CameraError(CameraErrorCode.inaccessible)
 
         if files_in_folder:
             # Distinguish the file type for every file in the folder
             names = [name for name, value in files_in_folder]
+            if '.nomedia' in names:
+                # do nothing with this folder
+                logging.debug("Ignoring %s because it contains a .nomedia file", path)
+                return
             split_names = [os.path.splitext(name) for name in names]
             # Remove the period from the extension
             exts = [ext[1:] for name, ext in split_names]
@@ -520,8 +583,8 @@ class ScanWorker(WorkerInPublishPullPipeline):
                     modification_time, size = self.camera.get_file_info(path, name)
                 except gp.GPhoto2Error as e:
                     logging.error(
-                        "Unable to access modification_time or size from %s on %s. Error code: %s",
-                        os.path.join(path, name), self.display_name, e.code
+                        "Unable to access modification_time or size from %s on %s. Error: %s",
+                        os.path.join(path, name), self.display_name, gphoto2_named_error(e.code)
                     )
                     modification_time, size = 0, 0
                     uri = get_uri(
@@ -604,9 +667,15 @@ class ScanWorker(WorkerInPublishPullPipeline):
                 if self.scan_preferences.scan_this_path(os.path.join(path, name)):
                     folders.append(name)
         except gp.GPhoto2Error as e:
-            logging.error("Unable to scan files on %s. Error code: %s", self.display_name, e.code)
+            logging.error(
+                "Unable to list folders on %s: %s", self.display_name,
+                gphoto2_named_error(e.code)
+            )
             uri = get_uri(path=path, camera_details=self.camera_details)
             self.problems.append(CameraDirectoryReadProblem(uri=uri, name=path, gp_code=e.code))
+            if e.code in (gp.GP_ERROR_IO_USB_FIND, gp.GP_ERROR_BAD_PARAMETERS):
+                logging.error("%s removed while listing folders during scan", self.display_name)
+                raise CameraError(code=CameraErrorCode.inaccessible)
 
         # recurse over subfolders
         for name in folders:
@@ -622,16 +691,21 @@ class ScanWorker(WorkerInPublishPullPipeline):
             files.sort(key=operator.attrgetter('size'))
 
         # When determining how a camera reports modification time, extraction order
-        # of preference is (1) jpeg, (2) RAW, and finally least preferred is (3) video
-        # However, if ignore_mdatatime_for_mtp_dng is set, ignore the RAW files
+        # of preference is (1) heif, (2) jpeg, (3) RAW, and finally least preferred
+        # is (4) video. However, if ignore_mdatatime_for_mtp_dng is set, ignore the RAW files
 
         if not self.ignore_mdatatime_for_mtp_dng:
-            order = (FileExtension.jpeg, FileExtension.raw, FileExtension.video)
+            order = (FileExtension.heif, FileExtension.jpeg, FileExtension.raw, FileExtension.video)
         else:
-            order = (FileExtension.jpeg, FileExtension.video, FileExtension.raw)
+            order = (FileExtension.heif, FileExtension.jpeg, FileExtension.video, FileExtension.raw)
+
+        if not fileformats.heif_capable():
+            order = order[1:]
 
         have_photos = len(self._camera_photos_videos_by_type[FileExtension.raw]) > 0 or \
                       len(self._camera_photos_videos_by_type[FileExtension.jpeg]) > 0
+        if not have_photos and fileformats.heif_capable():
+            have_photos = len(self._camera_photos_videos_by_type[FileExtension.heif]) > 0
         have_videos = len(self._camera_photos_videos_by_type[FileExtension.video]) > 0
 
         max_attempts = 5
@@ -757,9 +831,8 @@ class ScanWorker(WorkerInPublishPullPipeline):
                 adjusted_mtime = self.adjusted_mtime(modification_time)
 
                 downloaded = self.downloaded.file_downloaded(
-                                        name=self.file_name,
-                                        size=size,
-                                        modification_time=adjusted_mtime)
+                    name=self.file_name, size=size, modification_time=adjusted_mtime
+                )
 
                 thumbnail_cache_status = ThumbnailCacheDiskStatus.unknown
 
@@ -987,12 +1060,14 @@ class ScanWorker(WorkerInPublishPullPipeline):
                     self.problems.append(
                         CameraFileReadProblem(uri=uri, name=name, gp_code=e.gp_code)
                     )
-                else:
-                    assert e.code == CameraErrorCode.write
+                elif e.code == CameraErrorCode.write:
                     uri = get_uri(path=os.path.dirname(temp_name))
                     self.problems.append(
                         FileWriteProblem(uri=uri, name=temp_name, exception=e.py_exception)
                     )
+                else:
+                    if e.gp_code in (gp.GP_ERROR_IO_USB_FIND, gp.GP_ERROR_BAD_PARAMETERS):
+                        raise CameraError(code=CameraErrorCode.inaccessible)
             else:
                 if file_type == FileType.video:
                     metadata = metadatavideo.MetaData(temp_name, self.et_process)
@@ -1054,18 +1129,33 @@ class ScanWorker(WorkerInPublishPullPipeline):
 
         if ext_type == FileExtension.jpeg:
             determined_by = 'jpeg'
-            if self.camera.can_fetch_thumbnails:
-                use_app1 = True
-            else:
+            if self.prefs.force_exiftool:
                 exif_extract = True
+                use_exiftool = True
+                save_chunk = True
+            else:
+                if self.camera.can_fetch_thumbnails:
+                    use_app1 = True
+                else:
+                    exif_extract = True
+
         elif ext_type == FileExtension.raw:
             determined_by = 'RAW'
             exif_extract = True
-            use_exiftool = fileformats.use_exiftool_on_photo(extension)
+            use_exiftool = self.prefs.force_exiftool or fileformats.use_exiftool_on_photo(
+                extension, preview_extraction_irrelevant=True
+            )
             save_chunk = use_exiftool
         elif ext_type == FileExtension.video:
             determined_by = 'video'
             save_chunk = True
+        elif ext_type == FileExtension.heif:
+            determined_by = 'HEIF / HEIC'
+            exif_extract = True
+            use_exiftool = self.prefs.force_exiftool or fileformats.use_exiftool_on_photo(
+                extension, preview_extraction_irrelevant=True
+            )
+            save_chunk = True
 
         if use_app1:
             try:
@@ -1075,6 +1165,9 @@ class ScanWorker(WorkerInPublishPullPipeline):
                     full_file_name=os.path.join(path, name), camera_details=self.camera_details
                 )
                 self.problems.append(CameraFileReadProblem(uri=uri, name=name, gp_code=e.gp_code))
+                if e.gp_code in (gp.GP_ERROR_IO_USB_FIND, gp.GP_ERROR_BAD_PARAMETERS):
+                    raise CameraError(code=CameraErrorCode.inaccessible)
+
             else:
                 try:
                     with stdchannel_redirected(sys.stderr, os.devnull):
@@ -1111,7 +1204,13 @@ class ScanWorker(WorkerInPublishPullPipeline):
                 if offset is None:
                     offset = size
                 offset = min(size, offset)
-                self.sample_exif_bytes = self.camera.get_exif_extract(path, name, offset)
+                try:
+                    self.sample_exif_bytes = self.camera.get_exif_extract(path, name, offset)
+                except CameraProblemEx as e:
+                    self.sample_exif_bytes = None
+                    if e.gp_code in (gp.GP_ERROR_IO_USB_FIND, gp.GP_ERROR_BAD_PARAMETERS):
+                        raise CameraError(code=CameraErrorCode.inaccessible)
+
                 if self.sample_exif_bytes is not None:
                     try:
                         with stdchannel_redirected(sys.stderr, os.devnull):
@@ -1125,7 +1224,8 @@ class ScanWorker(WorkerInPublishPullPipeline):
                         )
                         self.sample_exif_bytes = None
                         uri = get_uri(
-                            full_file_name=os.path.join(path, name), camera_details=self.camera_details
+                            full_file_name=os.path.join(path, name),
+                            camera_details=self.camera_details
                         )
                         self.problems.append(FileMetadataLoadProblem(uri=uri, name=name))
                     else:
@@ -1146,7 +1246,6 @@ class ScanWorker(WorkerInPublishPullPipeline):
                 file_type=FileType.video
             )
 
-
         if dt is None:
             logging.warning(
                 "Scanner failed to extract date time metadata from %s on %s",
@@ -1181,6 +1280,8 @@ class ScanWorker(WorkerInPublishPullPipeline):
             determined_by = 'RAW'
         elif ext_type == FileExtension.video:
             determined_by = 'video'
+        elif ext_type == FileExtension.heif:
+            determined_by = 'HEIF / HEIC'
 
         if ext_type == FileExtension.video:
             metadata = metadatavideo.MetaData(
@@ -1190,7 +1291,9 @@ class ScanWorker(WorkerInPublishPullPipeline):
             dt = metadata.date_time(missing=None)
         else:
             # photo - we don't care if jpeg or RAW
-            if fileformats.use_exiftool_on_photo(extension):
+            if self.prefs.force_exiftool or fileformats.use_exiftool_on_photo(
+                    extension, preview_extraction_irrelevant=True):
+
                 metadata = metadataexiftool.MetadataExiftool(
                     full_file_name=full_file_name, et_process=self.et_process,
                     file_type=file_type
@@ -1264,12 +1367,13 @@ class ScanWorker(WorkerInPublishPullPipeline):
         Attempt to determine the device's approach to timezones when it
         store timestamps.
         When determining how this device reports modification time, file
-        preference is (1) RAW, (2)jpeg, and finally least preferred is (3)
-        video -- a RAW is the least likely to be modified.
+        preference is (1) RAW, (2) jpeg, (3) heif / heic, and finally least
+        preferred is (4) video -- a RAW is the least likely to be modified.
 
         NOTE: this creates a sample file for one type of file (RAW if present,
-        if not, then jpeg, if jpeg also not present, then video). However if
-        a raw / jpeg is found, then still need to create sample file for video.
+        if not, then jpeg, if jpeg also not present, then heif / heic, if that
+        not present, then video). However if a photo is found, then still need
+        to create a sample file for video.
         """
 
         logging.debug("Distinguishing approach to timestamp time zones on %s", self.display_name)
@@ -1278,13 +1382,22 @@ class ScanWorker(WorkerInPublishPullPipeline):
 
         max_attempts = 10
         raw_attempts = 0
-        jpegs_and_videos = defaultdict(deque)
+        jpegs_heifs_and_videos = defaultdict(deque)
+
+        # Only use HEIF files if we can read their metadata
+        if fileformats.heif_capable():
+            extensions = (
+                FileExtension.raw, FileExtension.jpeg, FileExtension.heif, FileExtension.video
+            )
+        else:
+            extensions = (FileExtension.raw, FileExtension.jpeg, FileExtension.video)
+        non_raw_extensions = extensions[1:]
 
         for dir_name, name in self.walk_file_system(path):
             full_file_name = os.path.join(dir_name, name)
             extension = fileformats.extract_extension(full_file_name)
             ext_type = fileformats.extension_type(extension)
-            if ext_type in (FileExtension.raw, FileExtension.jpeg, FileExtension.video):
+            if ext_type in extensions:
                 file_type = fileformats.file_type(extension)
                 if ext_type == FileExtension.raw and raw_attempts < max_attempts:
                     # examine right away
@@ -1294,17 +1407,17 @@ class ScanWorker(WorkerInPublishPullPipeline):
                             ext_type=ext_type, extension=extension, file_type=file_type):
                         return
                 else:
-                    if len(jpegs_and_videos[ext_type]) < max_attempts:
-                        jpegs_and_videos[ext_type].append(
+                    if len(jpegs_heifs_and_videos[ext_type]) < max_attempts:
+                        jpegs_heifs_and_videos[ext_type].append(
                             (dir_name, name, full_file_name, extension)
                         )
 
-                    if len(jpegs_and_videos[FileExtension.jpeg]) == max_attempts:
+                    if len(jpegs_heifs_and_videos[FileExtension.jpeg]) == max_attempts:
                         break
 
         # Couldn't locate sample raw file. Are left with up to max_attempts jpeg and video files
-        for ext_type in (FileExtension.jpeg, FileExtension.video):
-            for dir_name, name, full_file_name, extension in jpegs_and_videos[ext_type]:
+        for ext_type in non_raw_extensions:
+            for dir_name, name, full_file_name, extension in jpegs_heifs_and_videos[ext_type]:
                 file_type = fileformats.file_type(extension)
                 if self.examine_sample_non_camera_file(
                         dirname=dir_name, name=name, full_file_name=full_file_name,
@@ -1501,6 +1614,7 @@ def trace_lines(frame, event, arg):
     line_no = frame.f_lineno
     print('%s >>>>>>>>>>>>> At %s line %s' % (datetime.now().ctime(), func_name, line_no))
 
+
 def trace_calls(frame, event, arg):
     if event != 'call':
         return
@@ -1516,15 +1630,18 @@ def trace_calls(frame, event, arg):
         caller_filename = caller.f_code.co_filename
     else:
         caller_line_no = caller_filename = ''
-    print('% s Call to %s on line %s of %s from line %s of %s'  %
-        (datetime.now().ctime(), func_name, func_line_no, func_filename, caller_line_no,
-         caller_filename))
+    print(
+        '% s Call to %s on line %s of %s from line %s of %s' % (
+            datetime.now().ctime(), func_name, func_line_no, func_filename, caller_line_no,
+            caller_filename)
+    )
 
-    for f in ('distingish_non_camera_device_timestamp','determine_device_timestamp_tz'):
+    for f in ('distinguish_non_camera_device_timestamp','determine_device_timestamp_tz'):
         if func_name.find(f) >= 0:
             # Trace into this function
             return trace_lines
 
+
 if __name__ == "__main__":
     if os.getenv('RPD_SCAN_DEBUG') is not None:
         sys.settrace(trace_calls)
