@@ -21,6 +21,7 @@ __author__ = 'Damon Lynch'
 __copyright__ = "Copyright 2007-2020, Damon Lynch"
 
 import contextlib
+import site
 import locale
 import logging
 import os
@@ -107,14 +108,17 @@ def available_cpu_count(physical_only=False) -> int:
     available = None
     if sys.platform.startswith('linux'):
         try:
-            m = re.search(r'(?m)^Cpus_allowed:\s*(.*)$',
-                          open('/proc/self/status').read())
+            status_file = open('/proc/self/status')
+            status = status_file.read()
+            status_file.close()
+        except IOError:
+            pass
+        else:
+            m = re.search(r'(?m)^Cpus_allowed:\s*(.*)$', status)
             if m:
                 available = bin(int(m.group(1).replace(',', ''), 16)).count('1')
                 if available > 0 and not physical_only:
                     return available
-        except IOError:
-            pass
 
     if physical_only:
         physical = psutil.cpu_count(logical=False)
@@ -1220,23 +1224,27 @@ def available_languages(display_locale_code: str='') -> List[Tuple[str, str]]:
     return langs
 
 
-def installed_using_pip(package='rapid-photo-downloader') -> bool:
+def installed_using_pip(package: str, suppress_errors: bool = True) -> bool:
     """
-    Determine if python package was installed using pip.
+    Determine if python package was installed in local directory using pip.
+
+    Determination is not 100% robust in all circumstances.
 
     Exceptions are not caught.
 
     :param package: package name to search for
+    :param suppress_errors: if True, silently catch all exceptions and return False
     :return: True if installed via pip, else False
     """
 
-    pip_install = False
-    pkg = pkg_resources.get_distribution(package)
-    if pkg.has_metadata('INSTALLER'):
-        if pkg.get_metadata('INSTALLER').strip() == 'pip':
-            pip_install = True
-
-    return pip_install
+    try:
+        pkg = pkg_resources.get_distribution(package)
+        location = pkg.location
+        return not location.startswith('/usr') or location.find('local') > 0
+    except Exception:
+        if not suppress_errors:
+            raise
+        return False
 
 
 def getQtSystemTranslation(locale_name: str) -> Optional[QTranslator]:
